# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from propulsionai import PropulsionAI, AsyncPropulsionAI
from propulsionai.types.knowledgebase import KnowledgebaseItemResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestItem:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: PropulsionAI) -> None:
        item = client.knowledgebase.item.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
        )
        assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: PropulsionAI) -> None:
        item = client.knowledgebase.item.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
            metadata={},
        )
        assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: PropulsionAI) -> None:
        response = client.knowledgebase.item.with_raw_response.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        item = response.parse()
        assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: PropulsionAI) -> None:
        with client.knowledgebase.item.with_streaming_response.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            item = response.parse()
            assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: PropulsionAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledgebase_code` but received ''"):
            client.knowledgebase.item.with_raw_response.create(
                knowledgebase_code="",
                content="content",
                source="source",
            )


class TestAsyncItem:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPropulsionAI) -> None:
        item = await async_client.knowledgebase.item.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
        )
        assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPropulsionAI) -> None:
        item = await async_client.knowledgebase.item.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
            metadata={},
        )
        assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPropulsionAI) -> None:
        response = await async_client.knowledgebase.item.with_raw_response.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        item = await response.parse()
        assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPropulsionAI) -> None:
        async with async_client.knowledgebase.item.with_streaming_response.create(
            knowledgebase_code="knowledgebase_code",
            content="content",
            source="source",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            item = await response.parse()
            assert_matches_type(KnowledgebaseItemResponse, item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncPropulsionAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledgebase_code` but received ''"):
            await async_client.knowledgebase.item.with_raw_response.create(
                knowledgebase_code="",
                content="content",
                source="source",
            )
